#include "grading5.h"

#include <iostream>

std::vector<Weighted_grade> read_grades()
{
    std::vector<Weighted_grade> result;
    Weighted_grade wg;

    while (std::cin >> wg.grade && std::cin >> wg.weight) {
        result.push_back(wg);
    }

    return result;
}

// Computes the weighted sum of a vector of weighted grades; that is, the sum
// of each grade times its weight.
double weighted_sum(std::vector<Weighted_grade> grades)
{
    double sum = 0;

    for (Weighted_grade wg : grades) {
        if (wg.weight < 0) throw negative_weight();

        sum += wg.weight * wg.grade;
    }

    return sum;
}

// Computes the sum of all weights in a vector of weighted grades.
double total_weight(std::vector<Weighted_grade> grades)
{
    double sum = 0;

    for (Weighted_grade wg : grades) {
        sum += wg.weight;
    }

    return sum;
}

double weighted_sum_without(std::vector<Weighted_grade> grades,
                            ssize_t which)
{
    double dropped = grades[which].weight * grades[which].grade;
    return weighted_sum(grades) - dropped;
}

double total_weight_without(std::vector<Weighted_grade> grades, ssize_t which)
{
    double dropped = grades[which].weight;
    return total_weight(grades) - dropped;
}

double weighted_average_without(std::vector<Weighted_grade> grades, ssize_t which)
{
    double numerator = weighted_sum_without(grades, which);
    double denominator = total_weight_without(grades, which);

    if (denominator == 0) throw average_undefined();

    return numerator / denominator;
}

Omission_result find_best_omission(std::vector<Weighted_grade> grades)
{
    Omission_result best { 0.0, -1 };

    for (ssize_t i = 0; i < grades.size(); ++i) {
        try {
            double average = weighted_average_without(grades, i);
            if (average > best.average) {
                best.index = i;
                best.average = average;
            }
        } catch (average_undefined&) { }
    }

    if (best.index == -1) throw average_undefined();

    return best;
}

void print_omission_result(Omission_result result)
{
    std::cout << result.average << ' ' << result.index << '\n';
}

