#include "grading5.h"
#include <UnitTest++/UnitTest++.h>

using Grades = std::vector<Weighted_grade>;

const double eps = 0.0001;

TEST(Total_weight_without)
{
    Grades v1{{0, 3}};
    CHECK_EQUAL(0, total_weight_without(v1, 0));

    Grades v2{{0, 3}, {0, 4}, {0, 5}};
    CHECK_EQUAL(9, total_weight_without(v2, 0));
    CHECK_EQUAL(8, total_weight_without(v2, 1));
    CHECK_EQUAL(7, total_weight_without(v2, 2));
}

TEST(Weighted_sum_without)
{
    Grades v1{{1, 3}};
    CHECK_EQUAL(0, weighted_sum_without(v1, 0));

    Grades v2{{1, 3}, {2, 4}, {3, 5}};
    CHECK_EQUAL(23, weighted_sum_without(v2, 0));
    CHECK_EQUAL(18, weighted_sum_without(v2, 1));
    CHECK_EQUAL(11, weighted_sum_without(v2, 2));
}

TEST(Weighted_sum_without_throws)
{
    Grades v{{0, 3}, {0, -4}, {0, 5}};
    CHECK_THROW(weighted_sum_without(v, 0), negative_weight);
    CHECK_THROW(weighted_sum_without(v, 1), negative_weight);
    CHECK_THROW(weighted_sum_without(v, 2), negative_weight);
}

TEST(Weighted_average_without)
{
    Grades v1{{1, 3}};
    CHECK_EQUAL(0, weighted_sum_without(v1, 0));

    Grades v2{{1, 3}, {2, 4}, {3, 5}};
    CHECK_CLOSE(23.0 / 9, weighted_average_without(v2, 0), eps);
    CHECK_CLOSE(18.0 / 8, weighted_average_without(v2, 1), eps);
    CHECK_CLOSE(11.0 / 7, weighted_average_without(v2, 2), eps);
}

TEST(Weighted_average_without_throws)
{
    Grades v{{0, 3}, {0, -4}, {0, 5}};
    CHECK_THROW(weighted_average_without(v, 0), negative_weight);
    CHECK_THROW(weighted_average_without(v, 1), negative_weight);
    CHECK_THROW(weighted_average_without(v, 2), negative_weight);
}

TEST(Weighted_average_without_handles_zero_weight)
{
    Grades v{{1, 0}, {2, 0}, {3, 1}};

    CHECK_EQUAL(3, weighted_average_without(v, 0));
    CHECK_EQUAL(3, weighted_average_without(v, 1));
    CHECK_THROW(weighted_average_without(v, 2), average_undefined);
}

TEST(Find_best_omission)
{
    Grades v1{{90, 3}, {80, 2}};
    Omission_result o1{90, 1};
    CHECK_EQUAL(o1.index, find_best_omission(v1).index);
    CHECK_EQUAL(o1.average, find_best_omission(v1).average);
}

void check_fbo(Omission_result o, Grades v)
{
    CHECK_EQUAL(o.index, find_best_omission(v).index);
    CHECK_CLOSE(o.average, find_best_omission(v).average, eps);
}

TEST(Find_best_omission2)
{
    check_fbo({90, 1}, {{90, 3}, {80, 2}});
    check_fbo({90, 0}, {{80, 3}, {90, 2}});

    check_fbo({87.5, 0}, {{80, 3}, {90, 3}, {80, 1}});
    check_fbo({87.5, 1}, {{90, 3}, {80, 3}, {80, 1}});
    check_fbo({87.5, 2}, {{90, 3}, {80, 1}, {80, 3}});
}

TEST(Find_best_omission_throws)
{
    Grades v1{{}};
    CHECK_THROW(find_best_omission(v1), average_undefined);

    Grades v2{{10, 0}, {20, 0}};
    CHECK_THROW(find_best_omission(v2), average_undefined);
}

